package fr.username404.snowygui.config

import com.typesafe.config.Config
import com.typesafe.config.ConfigException
import com.typesafe.config.ConfigFactory
import com.typesafe.config.ConfigRenderOptions
import fr.username404.snowygui.Snowy
import io.github.config4k.extract
import net.minecraft.client.Minecraft
import java.io.File

object Configuration {
    private val file: File = File(Minecraft.getInstance().gameDirectory.absolutePath + File.separator + "snowy.conf")
    private val base = """
        |Snowy {
        |  displayInitMessage = true
        |}
    """.trimMargin()
    private val baseConf = ConfigFactory.parseString(base)
    @JvmField
    val obtained: Config = run {
        var result: Config = baseConf
        with(file) {
            if (!exists()) {
                createNewFile()
                setWritable(true)
                setReadable(true)
            } else try {
                result = ConfigFactory.parseFile(file)
            } catch (e: ConfigException) {
                Snowy.logs.warn("Could not parse the snowy configuration file, the default configuration will be used instead.")
            }
        }
        ConfigFactory.load(result).extract<Config>("Snowy").resolveWith(baseConf)
    }
    init {
        Runtime.getRuntime().addShutdownHook(
            Thread {
                file.writeText(
                    """
                        Snowy {
                          ${
                              obtained.root().render(
                                  ConfigRenderOptions.defaults()
                                      .setFormatted(true)
                                      .setJson(false)
                                      .setOriginComments(false)
                              ).trimEnd()
                          }
                        }
                    """.trimIndent()
                )
            }
        )
    }
}
